
# q-expansion of newform 6240.2.a.j, downloaded from the LMFDB on 03 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6240
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [1], [-1], [3], [-1], [-5], [2], [3], [0], [4], [1], [9], [-2], [-8], [-1], [-4], [-3], [16], [15], [-14], [15], [8], [-9], [-5], [-4], [-4], [-5], [20], [18], [8], [-12], [6], [1], [-5], [16], [-10], [-5], [8], [-18], [6], [1], [6], [17], [-10], [-16], [-8], [16], [-2], [4], [21], [7], [20], [0], [22], [0], [-4], [14], [-16], [18], [28], [24], [-21], [14], [-6], [-2], [18], [-18], [15], [-2], [-12], [0], [22], [14], [-24], [30], [36], [33], [18], [-8], [26], [-28], [40], [-14], [17], [-9], [-3], [23], [33], [-9], [-9], [-3], [-3], [-30], [-34], [24], [35], [30], [-2], [-38], [-32], [12], [19], [-2], [15], [-5], [-30], [8], [16], [-45], [2], [43], [-6], [-8], [12], [-2], [-49], [-27], [18], [-48], [-26], [12], [3], [10], [18], [-24], [36], [28], [14], [-15], [10], [42], [25], [-16], [-6], [-6], [0], [24], [21], [-2], [30], [-19], [24], [48], [-6], [-37], [39], [41], [-51], [-30], [6], [8], [-44], [-39], [-38], [30], [-29], [35], [0], [31], [-8], [-9], [-48], [34], [-42], [-24], [37], [-32], [-20], [-33], [-60], [2], [6], [-41], [21], [42], [40], [-17], [-16], [-26], [-49], [22], [-36], [-12], [-46], [-20], [18], [-19], [14], [5], [18], [26], [11], [-31], [-12], [-21], [-26], [44], [-60], [-41], [-18], [-25], [-17], [-31], [-27], [-3], [-2], [-20], [39], [41], [-52], [-48], [-8], [-60], [-32], [6], [-46], [28], [18], [30], [15], [-6], [-42], [-12], [-33], [-67], [41], [24], [10], [-9], [-2], [-11], [30], [0], [-26], [24], [-14], [-22], [46], [10], [-24], [-67], [64], [18], [10], [18], [-48], [23], [-48], [48], [-22], [22], [-68], [-31], [49], [-3], [10], [67], [10], [35], [-2], [53], [-40], [10], [72], [-29], [51], [7], [-7], [-50], [-62], [-43], [-37], [-44], [32], [29], [4], [32], [-38], [66], [19], [-56], [-72], [84], [20], [48], [26], [16], [-44], [-40], [-2], [-12], [-84], [12], [54], [46], [20], [-3], [6], [63], [-76], [18], [75], [-87], [12], [-19], [61], [19], [80], [40], [15], [-6], [84], [14], [-74], [53], [-15], [67], [90], [-12], [32], [38], [-30], [-1], [68], [-5], [4], [23], [63], [-70], [10], [-43], [40], [6], [56], [-56], [-20], [-64], [-72], [-2], [-5], [-23], [-77], [40], [46], [-86], [-79], [-33], [-24], [5], [32], [73], [-51], [-62], [11], [-46], [-21], [68], [54], [-70], [2], [-30], [-40], [44], [-30], [56], [88], [63], [-39], [16], [76], [22], [42], [53], [10], [-23], [52], [11], [-80], [-58], [-94], [-6], [-16], [-2], [-76], [-12], [-46], [-38], [-76], [3], [26], [-64], [7], [14], [-65], [22], [86], [12], [-15], [54], [-70], [-43], [45], [-2], [-11], [-43], [54], [34], [-3], [29], [6], [-43], [-41], [-43], [-32], [-60], [-51], [24], [76], [21], [-32], [-55], [78], [52], [-10], [-16], [43], [93], [-52], [74]]
